#include "Image.h"
#include "FloatPlane.h"
#include <cstring>
extern "C" {
#include <stdio.h>
#include <pam.h>
}

Image::Image()
{
    ready = false;
}


Image::Image(FloatPlane _plane[], const unsigned int _components)
{
    components = _components;
    plane = new FloatPlane[3];
    /* Copy FloatPlanes */
    for(unsigned int i=0; i<components; i++)
        plane[i] = _plane[i];
    ready = true;
}

void Image::setComponents(unsigned int n)
{
    if (ready == true)
        delete[] plane;

    components = n;

    plane = new FloatPlane[n];
    ready = true;
}

void Image::free()
{
    if (!ready)
        return;

    for(unsigned int i=0; i<components; i++)
        plane[i].free();

    delete[] plane;

    ready = false;
}

bool Image::sameDimensions()
{
    bool res = true;

    unsigned int width = plane[0].getWidth();
    unsigned int height = plane[0].getHeight();
    for(unsigned int i=1; i<components; i++)
    {
        if(plane[i].getWidth() != width || plane[i].getHeight() != height)
        {
            res = false;
            break;
        }
    }
    return res;
}

void Image::writePPM(const char *filename)
{
    if (components != 3)
        return;
    if (!sameDimensions())
        return;

    unsigned int width = plane[0].getWidth();
    unsigned int height = plane[0].getHeight();
    unsigned int MAXVALUE = plane[0].getMaxValue();

    struct pam outpam;
    unsigned int row;
    tuple *tuplerow;
 
    FILE* file;

    if (!ready)
        return;

    file = fopen(filename, "wb");
    if (file == NULL)
        return;


    outpam.size = sizeof(outpam);
    outpam.len = outpam.size;
    outpam.file = file;
    outpam.format = PPM_FORMAT;
    outpam.plainformat = 0; /* false */
    outpam.width = width;
    outpam.height = height;
    outpam.depth = 3; /* RGB - 3 sample per tuple */
    outpam.allocation_depth = 0; /* Same as depth. */
    outpam.maxval = MAXVALUE;
    std::strcpy(outpam.tuple_type, "RGB"); /* PNM non transparent */

    float *ptr[3];
    ptr[0] = plane[0].ptr;
    ptr[1] = plane[1].ptr;
    ptr[2] = plane[2].ptr;
 
    /* needs size,len,file,format,height,width,depth,maxval, tuple_type */
    pnm_writepaminit(&outpam);
 
    tuplerow = pnm_allocpamrow(&outpam);
 
    for (row = 0; row < height; row++) {
        unsigned int column;
        for (column = 0; column < width; ++column) {
            /* Only one plane */
            /* float to int! */
            for(unsigned int sample = 0; sample < components; ++sample)
                tuplerow[column][sample] = ptr[sample][column+width*row];
        }
        pnm_writepamrow(&outpam, tuplerow);
    }
 
    pnm_freepamrow(tuplerow);
    fclose(file);
}
