#include <assert.h>
#include "libjpegrest.h"


struct bitmap
bitmap_alloc(const int nplanes, const int width, const int height)
{
	struct bitmap bmap;
	int plane;

	bmap.m = malloc(sizeof(gsl_matrix *) * nplanes);
	assert(bmap.m != NULL);

	bmap.nplanes = nplanes;

	for (plane = 0; plane < bmap.nplanes; plane++)
	{
		bmap.m[plane] = gsl_matrix_alloc(height, width);
		assert(bmap.m[plane] != NULL);
	}

	bmap.is_ready = 1;

	return bmap;
}

void
bitmap_free(const struct bitmap bmap)
{
	int plane;

	for (plane = 0; plane < bmap.nplanes; plane++)
	{
		gsl_matrix_free(bmap.m[plane]);
	}
	free(bmap.m);
}

int are_bitmaps_comparable(const struct bitmap bmap1, const struct bitmap bmap2)
{
	int result = 1;
	int plane;

	if (bmap1.nplanes != bmap2.nplanes)
	{
		result = 0;
	} else
	{
		/* The sizes of each plane must be equal. */
		for (plane = 0; plane < bmap1.nplanes; plane++)
		{
			if (bmap1.m[plane]->size1 != bmap2.m[plane]->size1 ||
				bmap1.m[plane]->size2 != bmap2.m[plane]->size2)
			{
				result = 0;
				break;
			}
		}
	}

	return result;
}
