#include <stdio.h>
#include <pam.h>
#include "libjpegrest.h"

void bitmap_pam_init(int *argn, char **argv)
{
	pnm_init(argn, argv);
}

int
bitmap_filename_pam_get(const char *filename, struct bitmap *bmap)
{
	FILE *input;
	int res;

	/* Opening the FILE */
	input = fopen(filename,"r");
	if (input == NULL)
	{
		fprintf(stderr,"The file \"%s\" cannot be opened.\n",
			filename);
		res = -1;
	} else
	{
		res = bitmap_stdio_pam_get(input, bmap);
		fclose(input);
	}
	
	return res;
}

int
bitmap_stdio_pam_get(FILE *input, struct bitmap *bmap)
{
	struct pam inpam;
	tuple **tuples;
	int row, column, plane;


	/* Reading the input files PAM header */
	tuples = pnm_readpam(input, &inpam, PAM_STRUCT_SIZE(tuple_type));
	if (tuples == NULL)
	{
		fprintf(stderr, "Mem malloc error.");
		return -1;
	}

	/* Get the number of planes */
	*bmap = bitmap_alloc(inpam.depth, inpam.width, inpam.height);

	for (row = 0; row < inpam.height; row++) {
		for (column = 0; column < inpam.width; ++column) {
			for (plane = 0; plane < bmap->nplanes; plane++)
				bmap->m[plane]->data[row *
					bmap->m[plane]->tda + column] =
					tuples[row][column][plane];
		}
	}

	/* Free the pnm memory */
	pnm_freepamarray(tuples, &inpam);
	return 0;
}
