#include <gsl/gsl_matrix.h>
#include <jpeglib.h>
#include <stdio.h>


typedef float cblock[64];
typedef cblock qtable;
typedef cblock * cblockrow;
typedef cblockrow * cblockplane;

struct bjpeginfo
{
	int nplanes;
	int * widths_in_blocks;
	int * heights_in_blocks;
	cblockplane * blocks;  /* Image Block structure in planes */
	qtable * qtables; /* Qtables per plane */
	int is_ready; /* boolean. Is the block structure ready? */
	struct jpeg_decompress_struct srcinfo;
	JBLOCKIMAGE coefficients; /* [plane][row][column][coeficcient] */
	FILE *io_file;
};

struct bitmap
{
	int nplanes;
	int width;
	int height;
	gsl_matrix **m;  /* Pixels structure */
	int is_ready; /* boolean. Is the pixels structure ready? */
};

/* bitmap.c */
struct bitmap bitmap_alloc(const int nplanes, const int width,
		const int height);
void bitmap_free(const struct bitmap bmap);
int
are_bitmaps_comparable(const struct bitmap bmap1, const struct bitmap bmap2);

/* bjpeg.c */
void bjpeg_init(int *argn, char **argv);
struct bjpeginfo bjpeg_alloc(const int nplanes,
		const int * width_in_blocks,
		const int * height_in_blocks);
void bjpeg_free(struct bjpeginfo * jpeg);
void bjpeg_close(struct bjpeginfo *jpeg);
int bjpeg_filename_get(const char *filename, struct bjpeginfo *jpeg);
int bjpeg_stdio_get(FILE *input, struct bjpeginfo *jpeg);

/* bitmap_pam.c */
void bitmap_pam_init(int *argn, char **argv);
int bitmap_stdio_pam_get(FILE *input, struct bitmap *bmap);
int bitmap_filename_pam_get(const char *filename, struct bitmap *bmap);

/* bitmap_jpeg.c */
enum jpeg_conditions
{
	READ_RGB,
	READ_YCbCr,
	READ_PLANES,
	READ_COEFS
};

void bitmap_jpeg_init(int *argn, char **argv);
int bitmap_stdio_jpeg_get(FILE *input, struct bitmap *bmap,
	enum jpeg_conditions conds);
int bitmap_filename_jpeg_get(const char *filename, struct bitmap *bmap,
	enum jpeg_conditions conds);

/* error_measure.c */
float error_psnr(const gsl_matrix *m1, const gsl_matrix *m2);
float error_mse(const gsl_matrix *m1, const gsl_matrix *m2);
float error_se_variance(const gsl_matrix *m1, const gsl_matrix *m2);


/* error_gbim.c */
float error_mhgbim(const gsl_matrix *m, const float zeta);
float error_mvgbim(const gsl_matrix *m, const float zeta);
float error_mgbim_zeta(const gsl_matrix *m, const float zeta);
float error_mgbim(const gsl_matrix *m);

/* error_st.c */
void error_statistic(const gsl_matrix *m, double *mean, double *stddev);

/* colorc.c */
void rgb2ycc(const gsl_matrix *m0, const gsl_matrix *m1, const gsl_matrix *m2);
